# -*- coding: utf-8 -*-
from odoo import api, fields, models, _
from odoo.exceptions import ValidationError
from odoo.tools import float_is_zero
import time

class PecbPosSession(models.Model):
    _inherit = "pos.session"

    @api.multi
    def action_pos_session_closing_control(self):
        for session in self:
            if session.state == 'opened':
                lines = []
                for statement in session.statement_ids:
                    lines.append((0,0,{
                        'balance_end': statement.balance_end,
                        'balance_real': statement.balance_end,
                        'bank_statement_id': statement.id
                    }))
                session.write({'difference_lines': lines})
            if session.state == 'closing_control':
                config = session.config_id
                company_currency = config.company_id.currency_id
                move_lines = []
                for difference_line in session.difference_lines:
                    diff_amount = difference_line.difference
                    if difference_line.journal_id == difference_line.journal_dest_id:
                        raise ValidationError('El diario de ajuste no debe ser igual al diario original de extracto')
                    if not float_is_zero(diff_amount, precision_digits=session.currency_id.decimal_places) and difference_line.journal_dest_id:
                        credit_account_id = False
                        debit_account_id = False
                        journal_id = False
                        journal_from = False
                        journal_to = False
                        amount = session.currency_id.compute(diff_amount, company_currency)
                        if diff_amount < 0.0:
                            debit_account_id = difference_line.journal_dest_id.default_debit_account_id.id
                            credit_account_id = difference_line.journal_id.default_debit_account_id.id
                            journal_id = difference_line.journal_dest_id.id
                            journal_from = difference_line.journal_id
                            journal_to = difference_line.journal_dest_id
                        else:
                            debit_account_id = difference_line.journal_id.default_debit_account_id.id
                            credit_account_id = difference_line.journal_dest_id.default_debit_account_id.id
                            journal_id = difference_line.journal_id.id
                            journal_to = difference_line.journal_id
                            journal_from = difference_line.journal_dest_id
                        move_lines.append((0,0,{
                            'name': 'Ajuste (Incremento en %s)'%journal_to.name,
                            'account_id': debit_account_id,
                            'credit': 0.0,
                            'debit': abs(amount),
                            'journal_id': journal_id,
                            'currency_id': session.currency_id.id,
                            'amount_currency': company_currency != session.currency_id and abs(amount) or 0.0,
                        }))
                        move_lines.append((0,0,{
                            'name': 'Ajuste (Disminucion en %s)'%journal_from.name,
                            'account_id': credit_account_id,
                            'credit': abs(amount),
                            'debit': 0.0,
                            'journal_id': journal_id,
                            'currency_id': session.currency_id.id,
                            'amount_currency': company_currency != session.currency_id and abs(amount) or 0.0,
                        }))
                if move_lines:
                    move_vals = {
                        'ref': 'Ajuste diferencia %s'%session.name,
                        'date': time.strftime("%Y-%m-%d"),
                        'journal_id': journal_id,
                        'line_ids': move_lines,
                    }
                    move = self.env['account.move'].create(move_vals)
                    move.post()
        super(PecbPosSession, self).action_pos_session_closing_control()

    difference_lines = fields.One2many('pos.session.difference.line','pos_session_id','Ajuste de Diferencias', copy=False)

class PecbPosSessionDifference(models.Model):
    _name = "pos.session.difference.line"

    pos_session_id = fields.Many2one('pos.session', string="Sesión", copy=False) 
    bank_statement_id = fields.Many2one('account.bank.statement', string="Extracto Bancario", copy=False)
    journal_id = fields.Many2one('account.journal','Diario',related="bank_statement_id.journal_id",store=True)
    currency_id = fields.Many2one('res.currency','Moneda',related="bank_statement_id.currency_id",store=True)
    company_id = fields.Many2one('res.company','Compañía',related="bank_statement_id.company_id",store=True)
    balance_end = fields.Monetary('Balance Calculado')
    balance_real = fields.Monetary('Balance Real')
    difference = fields.Monetary('Diferencia', compute="_compute_difference", store=True)
    journal_dest_id = fields.Many2one('account.journal', 'Diario Ajuste', domain="[('journal_user','=',True)]")
    to_adjust = fields.Boolean('A ajustar',compute="compute_adjust",store=True)

    @api.multi
    @api.depends('journal_dest_id','difference')
    def compute_adjust(self):
        for line in self:
            if not float_is_zero(line.difference, precision_digits=line.pos_session_id.currency_id.decimal_places) \
                and line.journal_dest_id:
                line.to_adjust = True
            
    @api.multi
    @api.depends('balance_end','balance_real')
    def _compute_difference(self):
        for line in self:
            line.difference = line.balance_real - line.balance_end
